// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:wyatt_authentication_bloc/src/authentication/cubit/authentication_cubit.dart';
import 'package:wyatt_authentication_bloc/src/models/user/user_interface.dart';

class AuthenticationBuilder extends StatelessWidget {
  const AuthenticationBuilder({
    Key? key,
    required this.authenticated,
    required this.unauthenticated,
    required this.unknown,
  }) : super(key: key);

  final Widget Function(
    BuildContext context,
    UserInterface user,
    Map<String, dynamic>? userData,
  ) authenticated;
  final Widget Function(BuildContext context) unauthenticated;
  final Widget Function(BuildContext context) unknown;

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<AuthenticationCubit, AuthenticationState>(
      builder: (context, state) {
        if (state.status == AuthenticationStatus.authenticated) {
          if (state.user != null) {
            return authenticated(context, state.user!, state.userData);
          } else {
            return unauthenticated(context);
          }
        } else if (state.status == AuthenticationStatus.unauthenticated) {
          return unauthenticated(context);
        } else {
          return unknown(context);
        }
      },
    );
  }
}
