// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

part of prompt.framework;

class BuildContext {
  int _buildCount = 0;
  int get buildCount => _buildCount;
  void incrementBuildCount() => _buildCount++;
  void resetBuildCount() => _buildCount = 0;

  int _lineCount = 0;
  int get lineCount => _lineCount;
  void incrementLineCount() => _lineCount++;
  void resetLineCount() => _lineCount = 0;

  static void reset() {
    Sequences.unhideCursor();
    Sequences.write(''.reset());
  }

  void hideCursor() {
    Sequences.hideCursor();
  }

  void showCursor() {
    Sequences.unhideCursor();
  }

  void write(String text) {
    Sequences.write(text);
  }

  void writeln(String text) {
    incrementLineCount();
    Sequences.writeln(text);
  }

  void erasePreviousLine([int n = 1]) {
    for (int i = 0; i < n; i++) {
      Sequences.moveUp(1);
      Sequences.clearEntireLine();
    }
  }

  void wipe() {
    erasePreviousLine(lineCount);
    resetLineCount();
  }

  int readKey() => stdin.readByteSync();

  String readLine({
    String initialValue = '',
  }) {
    var buffer = initialValue;
    var index = buffer.length;

    Sequences.savePosition();

    if (buffer.isNotEmpty) {
      write(buffer);
    }

    while (true) {
      final key = readKey();
      if (index == buffer.length) {
        buffer += String.fromCharCode(key);
        index++;
      } else {
        buffer = buffer.substring(0, index) +
            String.fromCharCode(key) +
            buffer.substring(index);
        index++;
      }

      Sequences.restorePosition();
      Sequences.clearLineFromCursorToEnd();
      write(buffer);
    }
  }
}
