// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

part of 'form_data_cubit.dart';

@immutable
class FormDataState {
  final FormStatus status;
  final FormData data;
  final String? errorMessage;

  const FormDataState({
    required this.data,
    this.status = FormStatus.pure,
    this.errorMessage,
  });

  FormDataState copyWith({
    FormStatus? status,
    FormData? data,
    String? errorMessage,
  }) {
    return FormDataState(
      status: status ?? this.status,
      data: data ?? this.data,
      errorMessage: errorMessage ?? this.errorMessage,
    );
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is FormDataState &&
        other.status == status &&
        other.data == data &&
        other.errorMessage == errorMessage;
  }

  @override
  int get hashCode {
    return status.hashCode ^ data.hashCode ^ errorMessage.hashCode;
  }

  @override
  String toString() {
    return 'FormDataState(status: $status, data: $data, '
        'errorMessage: $errorMessage)';
  }
}
